
call sysproc.set_routine_opts('ISOLATION UR') @

CREATE OR REPLACE PROCEDURE INSERT_IVR_ALERT_ACTION (
    IN I_ALERT_ID	BIGINT,
    IN I_MEMO_TEXT	VARCHAR(1500),
    IN I_INVALID_CONTACT_DETAILS_IND	CHARACTER(1),
    IN I_CONTACT_HOME_PHONE_IND	CHARACTER(1),
    IN I_CONTACT_WORK_PHONE_IND	CHARACTER(1),
    IN I_CONTACT_MOBILE_PHONE_IND	CHARACTER(1),
    IN I_USER_NAME	CHARACTER(60),
    IN I_USER_ID	BIGINT,
    IN I_NEED_TO_COMMIT	CHARACTER(1),
    IN I_FULFILL_SENT_CODE	CHARACTER(1),
    OUT O_ALERT_AVAILABLE	CHARACTER(1),
    OUT O_SQLCODE	INTEGER,
    OUT O_SQLSTATE	CHARACTER(5) )
  SPECIFIC INSERT_IVR_ALERT_ACTION2
  LANGUAGE SQL
  NOT DETERMINISTIC
  EXTERNAL ACTION
  MODIFIES SQL DATA
  CALLED ON NULL INPUT
  INHERIT SPECIAL REGISTERS
-----------------------------------------------------------------------------
-- File:  INSERT_IVR_ALERT_ACTION
--
--
-- Description:
--
--
--
-- Called By:
--     n/a
--
-- Calls:
--      GET_ALERT_ROW
--      SAVE_ALERT_FULFIL
--
-- Notes:
--		Save an IVR alert action record
--      Called by the webservice, so we don't validate the users access to the alert
--
-- Usage:
--		Called from the web Service
--
--
-- History:
--     Date          User            Notes
--     2008/08/14    Stboyd		     
--     2008/11/20    leushe      included Alert_level column
--     2009/09/10   LEUSHE Added new parameter to pass to save_alert_fulfil I_MERCH_CNTRY_NAMES 
--     2010/03/30   LEUSHE  Removed I_MERCH_CNTRY_NAMES input var. 
--						    Changed the call to save_alert_Fulfill to match the new proc for alert fulfullments
--     2010/06/14   LEUSHE   changed the way the return is determined by checking alert suspend, queue_id, etc....
--     2010/07/14   LEUSHE   change the insert into alert action result_id to be coalesce(l_result_id,0) instead of
--							hardcoding to 0
--     2010/09/28   leushe  removed I_ALERT_VERSION as an input parameter and added support for USER_ID in the FCM_IVR_ALERT table query
--     2010/10/22   leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--     2010/10/29   leushe   changed the sql for determining if an alert is available. Updated the update_nbr in FCM_IVR_ALERT table for the alert_id
--     2010/12/13   leushe  added coalesce to coalesce(l_result_id,0)  < 6
--     2013/03/12    leushe      Change support to alert/entity values of 100 bytes
--     2014/10/05   leushe  Changed the servicable check to be coalesce(l_result_id,0)  < 7 and NEXT_SERVICE_TIMESTAMP < l_now
--
------------------------------------------------------------------------
BEGIN

	  	DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
	  	DECLARE SQLCODE  INT     DEFAULT 0;

    
		DECLARE l_alert_value CHAR(100);
	  	DECLARE   L_PARENT_ALERT_VALUE CHARACTER(100) ;
	  	DECLARE   L_PARENT_ALERT_TYPE CHARACTER(1) ;
	  	DECLARE   L_CONTACT_VALUE CHARACTER(100) ;
	  	DECLARE   L_CONTACT_TYPE CHARACTER(1) ;
		DECLARE   l_TENANT_multi_org_id integer;	  	
	  	DECLARE   l_alrt_ver_nbr integer ;
	  	DECLARE   l_strategy_id BIGINT ;
		DECLARE   l_queue_id BIGINT ;
		DECLARE   l_multi_org_id integer;
        DECLARE   l_alert_type char(1);		
		DECLARE   l_result_id  integer;
		DECLARE   l_alert_status character(10) ;
		DECLARE   l_gmt_offset decimal(3,1) DEFAULT 0.0;
		DECLARE   l_fraud_type_code  character(3);
		DECLARE   l_pend_reason_code character(3);
		DECLARE   l_fraud_score  integer;
		DECLARE   l_fraud_score_reason_code_1  integer;
		DECLARE   l_fraud_score_reason_code_2   integer;
		DECLARE   l_fraud_score_reason_code_3   integer;
		DECLARE   l_init_fraud_score integer;
		DECLARE   l_init_fraud_score_reason_code_1  integer;
		DECLARE   l_init_fraud_score_reason_code_2  integer;
		DECLARE   l_init_fraud_score_reason_code_3  integer;
		DECLARE   l_hwm_fraud_score  integer;
		DECLARE   l_hwm_fraud_score_reason_code_1   integer;
		DECLARE   l_hwm_fraud_score_reason_code_2   integer;
		DECLARE   l_hwm_fraud_score_reason_code_3   integer;
		DECLARE   l_alert_service_cnt  integer  ;
		DECLARE   l_alert_origin varchar(20) ;
		DECLARE   l_lst_transaction_amt  decimal(19,5);
		DECLARE   l_lst_transaction_timestamp  timestamp;
		DECLARE   l_user_service_timestamp timestamp;
		DECLARE   l_system_service_timestamp timestamp;
		DECLARE   l_next_service_timestamp timestamp;
		DECLARE   l_queue_entry_timestamp  timestamp;
		DECLARE   l_final_disposition_timestamp  timestamp;
		DECLARE   l_checkout_user  character(60) ;
		DECLARE   l_checkout_timestamp timestamp;
		DECLARE   l_create_user  character(60) ;
		DECLARE   l_create_timestamp timestamp;
		DECLARE   l_lstupdt_user character(60);
		DECLARE   l_lstupdt_timestamp  timestamp;
		DECLARE   l_alert_version  smallint ;
		DECLARE   l_logical_del_ind  character(1);
		DECLARE   l_action_type char(20) DEFAULT 'IVR ACTION';
		DECLARE   l_base_rule_id BIGINT ;
		DECLARE   l_rule_version_nbr integer ;
		DECLARE   l_pause_timestamp timestamp;
		DECLARE   l_pause_elapsed_time integer;
		DECLARE   l_alert_action_id BIGINT ;
		DECLARE l_sys_block_code_1 char(8);
		DECLARE l_trans_block_code char(8);
		DECLARE l_resurface_timestamp timestamp;
		DECLARE l_sent_secure_message_ind char(1) DEFAULT '0';
		DECLARE l_failed_contact_ind char(1);
		DECLARE l_invalid_contact_ind char(1);		
	    DECLARE l_now TIMESTAMP;


		SET l_now = current timestamp - current timezone;

	  -- Get copy of required alert_action field from the alert table

	  CALL GET_ALERT_ROW
         ( i_alert_id
          ,l_alrt_ver_nbr
          ,l_alert_value
          ,l_parent_alert_value
          ,l_parent_alert_type
          ,l_contact_value
          ,l_contact_type
          ,l_TENANT_multi_org_id
          ,l_queue_id
          ,l_multi_org_id
          ,l_alert_type
          ,l_result_id
          ,l_alert_status
          ,l_gmt_offset
          ,l_fraud_type_code
          ,l_fraud_score
          ,l_fraud_score_reason_code_1
          ,l_fraud_score_reason_code_2
          ,l_fraud_score_reason_code_3
          ,l_init_fraud_score
          ,l_init_fraud_score_reason_code_1
          ,l_init_fraud_score_reason_code_2
          ,l_init_fraud_score_reason_code_3
          ,l_hwm_fraud_score
          ,l_hwm_fraud_score_reason_code_1
          ,l_hwm_fraud_score_reason_code_2
          ,l_hwm_fraud_score_reason_code_3
          ,l_alert_service_cnt
          ,l_alert_origin
          ,l_lst_transaction_amt
          ,l_lst_transaction_timestamp
          ,l_user_service_timestamp
          ,l_system_service_timestamp
          ,l_next_service_timestamp
          ,l_queue_entry_timestamp
          ,l_final_disposition_timestamp
          ,l_checkout_user
          ,l_checkout_timestamp
          ,l_create_user
          ,l_create_timestamp
          ,l_lstupdt_user
          ,l_lstupdt_timestamp
          ,l_alert_version
          ,l_base_rule_id
          ,l_rule_version_nbr
          ,l_pause_timestamp
          ,l_pause_elapsed_time
          );

	  select strategy_id into l_strategy_id from fcm_queue where queue_id = l_queue_id;
	  -- Insert new alert action
	  INSERT INTO FCM_ALERT_ACTION(
					ALERT_ID,
					ALERT_VERSION_NBR,
					ACTION_TIMESTAMP,
					PARENT_ALERT_VALUE,
					PARENT_ALERT_TYPE,
					ALERT_VALUE,
					CONTACT_VALUE,
					CONTACT_TYPE,
					TENANT_multi_org_id,
					QUEUE_ID,
					MULTI_ORG_ID,
					ALERT_TYPE,
					RESULT_ID,
					ALERT_STATUS,
					GMT_OFFSET,
					FRAUD_TYPE_CODE,
					FRAUD_SCORE,
					FRAUD_SCORE_REASON_CODE_1,
					FRAUD_SCORE_REASON_CODE_2,
					FRAUD_SCORE_REASON_CODE_3,
					INIT_FRAUD_SCORE,
					INIT_FRAUD_SCORE_REASON_CODE_1,
					INIT_FRAUD_SCORE_REASON_CODE_2,
					INIT_FRAUD_SCORE_REASON_CODE_3,
					HWM_FRAUD_SCORE,
					HWM_FRAUD_SCORE_REASON_CODE_1,
					HWM_FRAUD_SCORE_REASON_CODE_2,
					HWM_FRAUD_SCORE_REASON_CODE_3,
					ALERT_SERVICE_CNT,
					ALERT_ORIGIN,
					LST_TRANSACTION_AMT,
					LST_TRANSACTION_TIMESTAMP,
					USER_SERVICE_TIMESTAMP,
					SYSTEM_SERVICE_TIMESTAMP,
					NEXT_SERVICE_TIMESTAMP,
					QUEUE_ENTRY_TIMESTAMP,
					FINAL_DISPOSITION_TIMESTAMP,
					CHECKOUT_USER,
					CHECKOUT_TIMESTAMP,
					CREATE_USER,
					CREATE_TIMESTAMP,
					LSTUPDT_USER,
					LSTUPDT_TIMESTAMP,
					LSTUPDT_NBR,
					CONTACT_HOME_PHONE_IND,
					CONTACT_MOBILE_PHONE_IND,
					CONTACT_WORK_PHONE_IND,
					--PEND_REASON_CODE,
					RESURFACE_TIMESTAMP,
					ACTION_TYPE,
					BASE_RULE_ID,
					RULE_VERSION_NBR,
					PAUSE_TIMESTAMP,
					PAUSE_ELAPSED_TIME,
					STRATEGY_ID
					)
					VALUES(
					i_alert_id,
					l_alrt_ver_nbr,
					l_now ,
					l_parent_alert_value,
					l_parent_alert_type,
					l_alert_value,
					l_contact_value,
					l_contact_type,	
					l_TENANT_multi_org_id,
					l_queue_id,
					l_multi_org_id,
					l_alert_type,
					0 ,-- result_id
					l_alert_status,
					l_gmt_offset,
					l_fraud_type_code ,
					l_fraud_score ,
					l_fraud_score_reason_code_1 ,
					l_fraud_score_reason_code_2 ,
					l_fraud_score_reason_code_3 ,
					l_init_fraud_score,
					l_init_fraud_score_reason_code_1,
					l_init_fraud_score_reason_code_2,
					l_init_fraud_score_reason_code_3,
					l_hwm_fraud_score ,
					l_hwm_fraud_score_reason_code_1 ,
					l_hwm_fraud_score_reason_code_2 ,
					l_hwm_fraud_score_reason_code_3 ,
					l_alert_service_cnt ,
					l_alert_origin,
					l_lst_transaction_amt ,
					l_lst_transaction_timestamp ,
					l_user_service_timestamp,
					l_system_service_timestamp,
					l_next_service_timestamp,
					l_queue_entry_timestamp ,
					l_final_disposition_timestamp ,
					l_checkout_user ,
					l_checkout_timestamp,
					i_user_name ,
					l_now,
					i_USER_NAME,
					l_now,
					l_alert_version,
					i_contact_home_phone_ind,
					i_contact_mobile_phone_ind,
					i_contact_work_phone_ind,
				--	l_pend_reason_code,
					l_resurface_timestamp,
					l_action_type,
					l_base_rule_id,
					l_rule_version_nbr,
					l_pause_timestamp,
					l_pause_elapsed_time,
					l_STRATEGY_ID
					);


	SET l_alert_action_id = identity_val_local();

	CALL SAVE_ALERT_FULFIL(
               l_alert_action_id,
               '0', -- i_SENT_LETTER_IND,
               '0', -- i_SENT_EMAIL_IND,
               '0',  -- i_SENT_SMS_IND,
               i_MEMO_TEXT
               ,i_USER_NAME
               ,l_SENT_SECURE_MESSAGE_IND
               ,i_INVALID_CONTACT_DETAILS_IND
               ,i_fulfill_sent_code
               ,NULL  --i_FULFIll_XML_MSG
               ,o_SQLCODE
               ,o_SQLSTATE
               );

	IF i_need_to_commit = 'Y' THEN
		COMMIT;
	END IF;

	SET o_sqlcode = 0;
	SET o_sqlstate = 'SUCESS';

	If (not exists (select 1 
				from fcm_alert_suspend fas 
				where l_contact_value = fas.contact_value 
				    and l_contact_type = fas.contact_type
				    and l_TENANT_multi_org_id = fas.TENANT_multi_org_id
					and fas.suspend_user !=i_user_name) 
			AND
    		    exists (select 1 
						from fcm_strategy_user su, fcm_queue q, fcm_strategy s
								where q.queue_id = l_queue_id and
								      su.strategy_id = q.strategy_id and
								      s.strategy_id = q.strategy_id and
								      s.strategy_status = 'ACTIVE' and
									  q.queue_status in ('ACTIVE', 'QUIESCE') and
								      su.user_id = i_user_id)
			AND
				l_alert_status = 'ACTIVE' 
			AND				
            	coalesce(l_result_id,0) < 7
            AND 
            	l_next_service_timestamp < l_now) THEN
	        
		SET o_alert_available = '1';
	ELSE
		SET o_alert_available = '0';
	END IF;
	update fcm_ivr_alert set update_nbr = update_nbr + 1 where alert_id = i_alert_id;
   END@

