
CALL SYSPROC.SET_ROUTINE_OPTS('ISOLATION UR') @

CREATE OR REPLACE PROCEDURE UNCHECKOUT_SPECIFIC_ALERT (
   IN I_ALERT_ID bigint, 
   IN I_USER_NAME CHARACTER (60)
)
   SPECIFIC UNCHECKOUT_SPECIFIC_ALERT
   LANGUAGE SQL
   NOT DETERMINISTIC
   EXTERNAL ACTION
   MODIFIES SQL DATA
   CALLED ON NULL INPUT
   INHERIT SPECIAL REGISTERS

-----------------------------------------------------------------------------
-- File:   uncheckout_alert.sql
--
--
-- Description:  procedure unchecks out a specific alert. It deletes the alert from the 
--               FCM_ALERT_SUSPEND table, if existing
-- 
--
-- Called By: uncheckout_alert
--
--
-- Calls:
--     None
--
-- Notes:
--
-- Usage:
--		I_ALERT_ID alert_id to be unchecked out
--   
--
-- History:     
--              2010/04/16   LEUSHE  Created 
--	            2010/05/24   LEUSHE  Added UNSUSPEND Alert Action insert
--              2010/10/22  leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively--
--				2010/12/12  leushe      Added 1 second to action_timestamp for Unsuspend alert action record because of unique constraint violation
--			    2011/03/30    leushe      Commented out the insert into FCM_ALERT_ACTION for SUSPEND actions
--              2011/04/01    leushe Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
--       		2013/03/12    leushe      Change support to alert/entity values of 100 bytes
--              2014/03/05    leushe Changed the query to check for alert_Suspend_cnt to use coalesce(max(alert_checkout_cnt),0) so there is always a result
--                                   and set the FCM_ALERT_ACTION.TENANT_MULTI_ORG_ID column    
--              2015/06/01   leushe  Moved Update to FCM_ALERT before Insert to FCM_ALERT_ACTION so LSTUPDT fields are correct in ALERT_ACTION
--
--
------------------------------------------------------------------------
UNCHK:
BEGIN
	
DECLARE   L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
DECLARE   L_CONTACT_VALUE  CHARACTER(100) ;
DECLARE   L_CONTACT_TYPE  CHARACTER(1) ;
DECLARE   L_TENANT_MULTI_ORG_ID  INTEGER ;

DECLARE   L_CNTR SMALLINT;

--Set the current timestamp once
			SET L_CURRENT_UTC_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE;
					
						UPDATE FCM_ALERT
						SET ALERT_STATUS = 'ACTIVE',
						  CHECKOUT_USER = NULL,
						  CHECKOUT_TIMESTAMP = NULL,
						  LSTUPDT_USER = I_USER_NAME,
						  LSTUPDT_TIMESTAMP = CURRENT_TIMESTAMP - CURRENT TIMEZONE,
						  LSTUPDT_NBR = LSTUPDT_NBR + 1
						WHERE ALERT_ID = I_ALERT_ID;

				INSERT INTO FCM_ALERT_ACTION
                          (ACTION_TIMESTAMP,
                           ACTION_TYPE,
                           ALERT_ID,
                           ALERT_VERSION_NBR,
                           PARENT_ALERT_VALUE,
                           PARENT_ALERT_TYPE,
                           ALERT_VALUE,
                           CONTACT_VALUE,
                           CONTACT_TYPE,
                           TENANT_MULTI_ORG_ID,
                           ALERT_TYPE,
                           QUEUE_ID,
                           MULTI_ORG_ID,
                           RESULT_ID,
                           ALERT_STATUS,
                           GMT_OFFSET,
                           FRAUD_TYPE_CODE,
                           FRAUD_SCORE,
                           FRAUD_SCORE_REASON_CODE_1,
                           FRAUD_SCORE_REASON_CODE_2,
                           FRAUD_SCORE_REASON_CODE_3,
                           INIT_FRAUD_SCORE,
                           INIT_FRAUD_SCORE_REASON_CODE_1,
                           INIT_FRAUD_SCORE_REASON_CODE_2,
                           INIT_FRAUD_SCORE_REASON_CODE_3,
                           HWM_FRAUD_SCORE,
                           HWM_FRAUD_SCORE_REASON_CODE_1,
                           HWM_FRAUD_SCORE_REASON_CODE_2,
                           HWM_FRAUD_SCORE_REASON_CODE_3,
                           ALERT_SERVICE_CNT,
                           ALERT_ORIGIN,
                           INIT_TRANSACTION_AMT,
                           INIT_TRANSACTION_TIMESTAMP,
                           LST_TRANSACTION_AMT,
                           LST_TRANSACTION_TIMESTAMP,
                           USER_SERVICE_TIMESTAMP,
                           SYSTEM_SERVICE_TIMESTAMP,
                           NEXT_SERVICE_TIMESTAMP,
                           QUEUE_ENTRY_TIMESTAMP,
                           FINAL_DISPOSITION_TIMESTAMP,
                           CHECKOUT_USER,
                           CHECKOUT_TIMESTAMP,
                           CREATE_USER,
                           CREATE_TIMESTAMP,
                           LSTUPDT_USER,
                           LSTUPDT_TIMESTAMP,
                           LSTUPDT_NBR,
                           BASE_RULE_ID,
                           RULE_VERSION_NBR)
						SELECT L_CURRENT_UTC_TIMESTAMP AS ACTION_TIMESTAMP, 
							'AUTO UNCHECKOUT'  AS ACTION_TYPE,
							ALERT_ID, ALERT_VERSION_NBR,parent_ALERT_VALUE,PARENT_ALERT_TYPE, ALERT_VALUE, 
							CONTACT_VALUE,CONTACT_TYPE, TENANT_MULTI_ORG_ID,ALERT_TYPE, QUEUE_ID,
							MULTI_ORG_ID, RESULT_ID, 'ACTIVE',
							GMT_OFFSET,
							FRAUD_TYPE_CODE,  
							FRAUD_SCORE,
							FRAUD_SCORE_REASON_CODE_1, FRAUD_SCORE_REASON_CODE_2,
							FRAUD_SCORE_REASON_CODE_3, INIT_FRAUD_SCORE,
							INIT_FRAUD_SCORE_REASON_CODE_1,
							INIT_FRAUD_SCORE_REASON_CODE_2,
							INIT_FRAUD_SCORE_REASON_CODE_3, HWM_FRAUD_SCORE,
							HWM_FRAUD_SCORE_REASON_CODE_1,
							HWM_FRAUD_SCORE_REASON_CODE_2,
							HWM_FRAUD_SCORE_REASON_CODE_3, ALERT_SERVICE_CNT,
							ALERT_ORIGIN, INIT_TRANSACTION_AMT,
							INIT_TRANSACTION_TIMESTAMP, LST_TRANSACTION_AMT,
							LST_TRANSACTION_TIMESTAMP, USER_SERVICE_TIMESTAMP,
							SYSTEM_SERVICE_TIMESTAMP, NEXT_SERVICE_TIMESTAMP,
							QUEUE_ENTRY_TIMESTAMP, FINAL_DISPOSITION_TIMESTAMP,
							CHECKOUT_USER, CHECKOUT_TIMESTAMP, CREATE_USER,
							CREATE_TIMESTAMP, LSTUPDT_USER, LSTUPDT_TIMESTAMP,
						  LSTUPDT_NBR, BASE_RULE_ID, RULE_VERSION_NBR
						FROM FCM_ALERT
						WHERE ALERT_ID = I_ALERT_ID;

			
	

/* commented out for race condition*/            
			            Select contact_value,contact_type,TENANT_multi_org_id into L_CONTACT_VALUE,l_contact_Type,l_TENANT_multi_org_id from fcm_alert where alert_id =  I_ALERT_ID;
			            /* Have to use max, because if no records, or alert_suspend is not on, it will error out*/	
			            Select coalesce(max(ALERT_CHECKOUT_CNT),0) into L_CNTR from fcm_alert_suspend 
			            	where contact_value = L_CONTACT_VALUE 
			            			and contact_Type = l_contact_type
			            			and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						If (L_CNTR = 1) then
							delete from fcm_alert_suspend 
								where contact_value = L_CONTACT_VALUE and contact_Type = l_contact_type and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						else if (L_CNTR > 1) THEN
							update fcm_alert_suspend
						  		set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT -1 
						  		where contact_value = L_CONTACT_VALUE and contact_Type = l_contact_type and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						  end if;
						end if;
			 --						DELETE FROM FCM_ALERT_SUSPEND WHERE ALERT_ID = I_ALERT_ID; 
						-- step 4. commit work;
						COMMIT;

END UNCHK @

CREATE OR REPLACE PROCEDURE UNCHECKOUT_ALERT (
   IN I_USER_NAME CHARACTER (60)
)
   SPECIFIC UNCHECKOUT_ALERT
   LANGUAGE SQL
   NOT DETERMINISTIC
   EXTERNAL ACTION
   MODIFIES SQL DATA
   CALLED ON NULL INPUT
   INHERIT SPECIAL REGISTERS

-----------------------------------------------------------------------------
-- File:   uncheckout_alert.sql
--
--
-- Description:  procedure periodically unchecksout abandoned alerts.
-- 
--
-- Called By: batch scheduler
--
--
-- Calls:
--     UNCHECKOUT_SPECIFIC_ALERT
--
-- Notes:
--
-- Usage:
--   
--
-- History:     01/03/2006:  created by Mike Ames (x10866 michael.ames@sas.com )
--              06/20/2006:  timezone mods
--              04/11/2007:  defect 19269 updates
--              06/19/2007:  adapted for R2 (initial pass)
--              07/18/2007:  Further adaptation for R2 data model changes.
--              11/27/2007:  added cma_acct_num to the insert statement
--              05/21/2008:  Modified the proc to handle IVR related uncheckouts
--             18DEC2008    LEUSHE  S0553598 Removed checking on the SQLSTATE variable. Changed this by setting 
--                                  the v_sqlstate variable SQLSTATE, the conditionally checking v_sqlstate.
--                                  In db2 9.5, accessing the SQLSTATE variable, resets it. Thus, the IF would reset the SQLSTATE 
--                                  value. 
--              10/05/2009:  Modified the proc to handle the case when a NON-IVR uncheckout situation, so that only non-IVR checkedout alerts
--                           are unchecked out.
--              2010/03/30   LEUSHE  Removed pend reason code from the alert_action insert. 
--              2010/04/26   LEUSHE  Modified to UNCHECKOUT_SPECIFIC_ALERT to perform the checkout
--              2010/06/14   LEUSHE  Since IVR alerts aren't checked out, no need to check if they are IVR or not
--
------------------------------------------------------------------------
UNCHK:
BEGIN

-- step 0. declare local variables

DECLARE L_UNCHK_DUR         INT;

DECLARE L_DEFAULT_INTERVAL  VARCHAR (10) DEFAULT '60' ;
DECLARE L_ALERT_ID          BIGINT DEFAULT -1 ;
--DECLARE SQLCODE             INTEGER DEFAULT 0 ;
DECLARE V_SQLSTATE          CHAR (5) DEFAULT '00000' ;
DECLARE SQLSTATE            CHAR (5) DEFAULT '00000' ;
			
DECLARE L_STMTTEMPLATE VARCHAR(1000);
			
DECLARE L_STMT STATEMENT;
DECLARE C_STMT CURSOR WITH HOLD FOR L_STMT;


-- step 1. set local variables


      SET L_UNCHK_DUR =(SELECT CAST( (CASE WHEN TRIM (PROPERTY_VALUE) = '' THEN L_DEFAULT_INTERVAL ELSE PROPERTY_VALUE END) AS INT)
										FROM FCM_PROPERTY
										WHERE PROPERTY_NAME = 'analyst.alert.uncheckout.interval');
 			
	SET L_STMTTEMPLATE = ' SELECT alert_id FROM fcm_alert WHERE ALERT_STATUS = ''CHECKOUT''  AND sysfun.timestampdiff(4,char ((CURRENT TIMESTAMP - current timezone) - checkout_timestamp)) > ? FOR READ ONLY';

      


      PREPARE L_STMT FROM L_STMTTEMPLATE;
      OPEN C_STMT USING L_UNCHK_DUR;
	
	  
      SET V_SQLSTATE = SQLSTATE;
      WHILE (V_SQLSTATE = '00000')
      DO

         FETCH C_STMT INTO L_ALERT_ID;
         SET V_SQLSTATE = SQLSTATE;
         IF (V_SQLSTATE = '00000')         THEN
				call UNCHECKOUT_SPECIFIC_ALERT (l_alert_id,i_user_name);
		 END IF;

      END WHILE;

--END IF;
     



END UNCHK @

