CREATE OR REPLACE PROCEDURE CR_MAN_ALERT (
     I_ALERT_VALUE IN VARCHAR2,
     I_PARENT_ALERT_VALUE IN VARCHAR2,
     I_PARENT_ALERT_TYPE IN VARCHAR2,
     I_CONTACT_VALUE IN VARCHAR2,
     I_CONTACT_TYPE IN VARCHAR2,     
     I_QUEUE_ID IN NUMBER,
     I_USER_NAME IN VARCHAR2,
     I_GROUP_CODE IN VARCHAR2,
     I_MULTI_ORG_NAME IN VARCHAR2,
     I_ALERT_TYPE IN VARCHAR2,
     I_NEED_TO_COMMIT IN VARCHAR2,
     o_getalertcur OUT SYS_REFCURSOR)
-----------------------------------------------------------------------------
-- file:   cr_man_alert
--
--
-- description:  creates a manual alert, needed for inbound when alert doesn't exist.
--
--
-- called by: user interface inbound
--
--
-- calls:
--	   get_alert_row
--     get_alert
--     save_alert_action
--
-- notes:
--
--
-- usage:
--
--
-- History:    
--     Date          User       Notes
--     2012/07/23    leushe		Ported from DB2
--     2013/03/12    leushe     Change support to alert/entity values of 100 bytes
--     2014/10/24    leushe     Fixed missing reference to TENANT_MULTI_ORG_ID for insert into FCM_ALERT_SUSPEND when
--                              existing Alert with Status CLOSED
--  
------------------------------------------------------------------------
IS

-- step 0. declare local variables

   L_MULTI_ORG_ID  INTEGER DEFAULT NULL;
   L_TENANT_MULTI_ORG_ID  INTEGER ;   
   L_GMT_OFFSET NUMBER(3,1) DEFAULT 0.0;
   L_ALERT_TYPE VARCHAR2(1);
   L_ALERT_ID NUMBER;
   L_ALRT_VER_NBR INTEGER;
   L_ALERT_VALUE  VARCHAR2(100);
   L_PARENT_ALERT_VALUE  VARCHAR2(100);
   L_CONTACT_VALUE  VARCHAR2(100);
   L_PARENT_ALERT_TYPE  VARCHAR2(1);
   L_CONTACT_TYPE  VARCHAR2(1);
   L_QUEUE_ID NUMBER;
   L_RESULT_ID  INTEGER;
   L_ALERT_STATUS VARCHAR2(10);
   L_FRAUD_TYPE_CODE  VARCHAR2(3);
   L_PEND_REASON_CODE VARCHAR2(20);
   L_FRAUD_SCORE  INTEGER;
   L_FRAUD_SCORE_REASON_CODE_1  INTEGER;
   L_FRAUD_SCORE_REASON_CODE_2   INTEGER;
   L_FRAUD_SCORE_REASON_CODE_3   INTEGER;
   L_INIT_FRAUD_SCORE INTEGER;
   L_INIT_FRD_SCORE_REASON_CODE_1  INTEGER;
   L_INIT_FRD_SCORE_REASON_CODE_2  INTEGER;
   L_INIT_FRD_SCORE_REASON_CODE_3  INTEGER;
   L_HWM_FRAUD_SCORE  INTEGER;
   L_HWM_FRD_SCORE_REASON_CODE_1   INTEGER;
   L_HWM_FRD_SCORE_REASON_CODE_2   INTEGER;
   L_HWM_FRD_SCORE_REASON_CODE_3   INTEGER;
   L_ALERT_SERVICE_CNT  INTEGER;
   L_ALERT_ORIGIN VARCHAR2(20);
   L_LST_TRANSACTION_AMT  BINARY_DOUBLE;
   L_LST_TRANSACTION_TIMESTAMP  TIMESTAMP;
   L_USER_SERVICE_TIMESTAMP TIMESTAMP;
   L_SYSTEM_SERVICE_TIMESTAMP TIMESTAMP;
   L_NEXT_SERVICE_TIMESTAMP TIMESTAMP;
   L_QUEUE_ENTRY_TIMESTAMP  TIMESTAMP;
   L_FINAL_DISPOSITION_TIMESTAMP  TIMESTAMP;
   L_CHECKOUT_USER  VARCHAR2(60);
   L_CHECKOUT_TIMESTAMP TIMESTAMP;
   L_CREATE_USER  VARCHAR2(60);
   L_CREATE_TIMESTAMP TIMESTAMP;
   L_LSTUPDT_USER VARCHAR2(60);
   L_LSTUPDT_TIMESTAMP  TIMESTAMP;
   L_LSTUPDT_NBR  SMALLINT;
   L_LOGICAL_DEL_IND  VARCHAR2(1);
   L_BASE_RULE_ID NUMBER;
   L_RULE_VERSION_NBR INTEGER;
   L_STRATEGY_ID NUMBER;
   L_PAUSE_TIMESTAMP  TIMESTAMP;
   L_PAUSE_ELAPSED_TIME INTEGER;
   L_ACTION_TYPE VARCHAR2(20);
   L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
   P_ALERT_ID NUMBER;
   L_SUSPEND VARCHAR2(5) DEFAULT 'false';
   L_ERROR INT DEFAULT -1;
   L_ROW_COUNT INT DEFAULT -1;
   L_USER_NAME VARCHAR2(60);

   CURSOR C_ALERTEXISTCURSOR  IS
    SELECT COALESCE(ALERT_ID,-1), ALERT_STATUS,CONTACT_VALUE,CONTACT_TYPE
    FROM FCM_ALERT
    WHERE ALERT_VALUE = I_ALERT_VALUE
    AND ALERT_TYPE = L_ALERT_TYPE;

BEGIN
L_USER_NAME:=I_USER_NAME;
--
--Check to see if suspended alerts is on
--
SELECT coalesce(SUBSTR(LOWER(PROPERTY_VALUE),1,5),'false')   INTO L_SUSPEND FROM FCM_PROPERTY WHERE PROPERTY_NAME =
        'analyst.alert.suspend.enabled';

    IF (L_SUSPEND is null) THEN
         L_SUSPEND := 'false';
    END IF;

--
--Check for a valid alert_type
--
SELECT ALERT_TYPE
INTO L_ALERT_TYPE
FROM FCM_ALERT_TYPE
WHERE ALERT_TYPE = I_ALERT_TYPE;

--
--If alert_type is not valid, return 0
--
IF ((L_ALERT_TYPE IS NULL) OR (LENGTH(TRIM(L_ALERT_TYPE)) = 0)) THEN
    RETURN ;
END IF;

--
--Check for a valid Multi_org. If not, use root multi_org_id
--

SELECT MULTI_ORG_ID,TENANT_MULTI_ORG_ID
		INTO L_MULTI_ORG_ID,l_TENANT_multi_org_id
        FROM FCM_MULTI_ORG
        WHERE MULTI_ORG_NAME = I_MULTI_ORG_NAME;

IF ( L_MULTI_ORG_ID IS NULL ) THEN
        SELECT MULTI_ORG_ID,TENANT_MULTI_ORG_ID
		INTO L_MULTI_ORG_ID,l_TENANT_multi_org_id
        FROM FCM_MULTI_ORG
        WHERE PARENT_MULTI_ORG_ID IS NULL;
END IF;

 	    
--
--Check Security Access to the Alert Type and Multi-org. If no access, send empty result set
--
IF (CHECK_USR_MO_AT_ACCESS(I_USER_NAME,L_MULTI_ORG_ID,L_ALERT_TYPE)=1)
THEN
            --
            -- check for null input variables for i_alert_value ,  i_alert_type and i_cust_num
            --
            IF ((I_ALERT_VALUE IS NULL) OR (LENGTH(TRIM(I_ALERT_VALUE)) = 0)) THEN
            RETURN ;
            END IF;
            
            IF ((I_CONTACT_VALUE IS NULL) OR (LENGTH(TRIM(I_CONTACT_VALUE)) = 0)) THEN
            RETURN ;
            END IF;
            IF ((I_CONTACT_TYPE IS NULL) OR (LENGTH(TRIM(I_CONTACT_TYPE)) = 0)) THEN
            RETURN ;
            END IF;            
            IF ((I_ALERT_TYPE IS NULL) OR (LENGTH(TRIM(I_ALERT_TYPE)) = 0)) THEN
            RETURN ;
            END IF;

            

            
            -- 2007/09/25 changes
            -- step 1a. get  gmt offset
            -- Default to 0, if the customer is not found
            
            /* Using coalesce(sum) because without an aggregate function, there is not result set. The aggregate function
             * forces a resultset of null, so we coalesce to get a value. Note there is only one record for a contact value and contact type
             * because of a unique index on the 2 columns*/	 
             SELECT
                COALESCE(SUM(GMT_OFFSET),0.0) INTO L_GMT_OFFSET
            FROM FCM_CONTACT
            WHERE CONTACT_VALUE = I_CONTACT_VALUE AND CONTACT_TYPE=I_CONTACT_TYPE and TENANT_MULTI_ORG_ID = L_TENANT_MULTI_ORG_ID;
            --select get_contact_offset(i_contact_Value,i_contact_type) into  L_GMT_OFFSET from dual;


            
            --Set the current timestamp once
             L_CURRENT_UTC_TIMESTAMP := SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);
            
            OPEN C_ALERTEXISTCURSOR ;
            FETCH  C_ALERTEXISTCURSOR INTO L_ALERT_ID, L_ALERT_STATUS,L_CONTACT_VALUE,L_CONTACT_TYPE;
            CLOSE C_ALERTEXISTCURSOR;
            
            IF ( L_ALERT_ID != -1 ) THEN
                    IF ( L_ALERT_STATUS = 'ACTIVE' ) THEN
                            --
                            -- in this case should a manual alert supercede an automated one?
                            -- we probably have much more info on the alert that was created from the ose
                            --    ( a queue, fraud score, fraud reason, ect...)
                            -- current alert should be left unchanged at this point
                            --
                             P_ALERT_ID := L_ALERT_ID;
            
                    ELSIF ( L_ALERT_STATUS = 'CHECKOUT' ) THEN
                                --
                                -- write alert action row
                                --
                                 GET_ALERT_ROW
                                    ( L_ALERT_ID
                                        ,L_ALRT_VER_NBR
                                        ,L_ALERT_VALUE
                                        ,L_PARENT_ALERT_VALUE
                                        ,L_PARENT_ALERT_TYPE
                                        ,L_CONTACT_VALUE
                                        ,L_CONTACT_TYPE
                                        ,l_TENANT_multi_org_id
                                        ,L_QUEUE_ID
                                        ,L_MULTI_ORG_ID
                                        ,L_ALERT_TYPE
                                        ,L_RESULT_ID
                                        ,L_ALERT_STATUS
                                        ,L_GMT_OFFSET
                                        ,L_FRAUD_TYPE_CODE
                                        ,L_FRAUD_SCORE
                                        ,L_FRAUD_SCORE_REASON_CODE_1
                                        ,L_FRAUD_SCORE_REASON_CODE_2
                                        ,L_FRAUD_SCORE_REASON_CODE_3
                                        ,L_INIT_FRAUD_SCORE
                                        ,L_INIT_FRD_SCORE_REASON_CODE_1
                                        ,L_INIT_FRD_SCORE_REASON_CODE_2
                                        ,L_INIT_FRD_SCORE_REASON_CODE_3
                                        ,L_HWM_FRAUD_SCORE
                                        ,L_HWM_FRD_SCORE_REASON_CODE_1
                                        ,L_HWM_FRD_SCORE_REASON_CODE_2
                                        ,L_HWM_FRD_SCORE_REASON_CODE_3
                                        ,L_ALERT_SERVICE_CNT
                                        ,L_ALERT_ORIGIN
                                        ,L_LST_TRANSACTION_AMT
                                        ,L_LST_TRANSACTION_TIMESTAMP
                                        ,L_USER_SERVICE_TIMESTAMP
                                        ,L_SYSTEM_SERVICE_TIMESTAMP
                                        ,L_NEXT_SERVICE_TIMESTAMP
                                        ,L_QUEUE_ENTRY_TIMESTAMP
                                        ,L_FINAL_DISPOSITION_TIMESTAMP
                                        ,L_CHECKOUT_USER
                                        ,L_CHECKOUT_TIMESTAMP
                                        , L_USER_NAME
                                        ,L_CREATE_TIMESTAMP
                                        , L_USER_NAME
                                        ,L_LSTUPDT_TIMESTAMP
                                        ,L_LSTUPDT_NBR
                                        ,L_BASE_RULE_ID
                                        ,L_RULE_VERSION_NBR
                                        ,L_PAUSE_TIMESTAMP
                                        ,L_PAUSE_ELAPSED_TIME
                                        );
                    
                     SAVE_ALERT_ACTION
                                        ( L_ALERT_ID
                                        ,L_ALRT_VER_NBR
                                        ,L_ALERT_VALUE
                                        ,L_PARENT_ALERT_VALUE
                                        ,L_PARENT_ALERT_TYPE
                                        ,L_CONTACT_VALUE
                                        ,L_CONTACT_TYPE
                                        ,l_TENANT_multi_org_id
                                        ,L_QUEUE_ID
                                        ,L_MULTI_ORG_ID
                                        ,L_ALERT_TYPE
                                        ,L_RESULT_ID
                                        ,L_ALERT_STATUS
                                        ,L_GMT_OFFSET
                                        ,L_FRAUD_TYPE_CODE
                                        ,L_FRAUD_SCORE
                                        ,L_FRAUD_SCORE_REASON_CODE_1
                                        ,L_FRAUD_SCORE_REASON_CODE_2
                                        ,L_FRAUD_SCORE_REASON_CODE_3
                                        ,L_INIT_FRAUD_SCORE
                                        ,L_INIT_FRD_SCORE_REASON_CODE_1
                                        ,L_INIT_FRD_SCORE_REASON_CODE_2
                                        ,L_INIT_FRD_SCORE_REASON_CODE_3
                                        ,L_HWM_FRAUD_SCORE
                                        ,L_HWM_FRD_SCORE_REASON_CODE_1
                                        ,L_HWM_FRD_SCORE_REASON_CODE_2
                                        ,L_HWM_FRD_SCORE_REASON_CODE_3
                                        ,L_ALERT_SERVICE_CNT
                                        ,L_ALERT_ORIGIN
                                        ,L_LST_TRANSACTION_AMT
                                        ,L_LST_TRANSACTION_TIMESTAMP
                                        ,L_USER_SERVICE_TIMESTAMP
                                        ,L_SYSTEM_SERVICE_TIMESTAMP
                                        ,L_NEXT_SERVICE_TIMESTAMP
                                        ,L_QUEUE_ENTRY_TIMESTAMP
                                        ,L_FINAL_DISPOSITION_TIMESTAMP
                                        ,L_CHECKOUT_USER
                                        ,L_CHECKOUT_TIMESTAMP
                                        ,I_USER_NAME
                                        ,L_CREATE_TIMESTAMP
                                        ,I_USER_NAME
                                        ,L_LSTUPDT_TIMESTAMP
                                        ,L_LSTUPDT_NBR
                                --        ,NULL --,l_pend_reason_code
                                        ,NULL --,l_reroute_queue_id
                                        ,'INB' -- ,l_service_method_code
                                        ,'ALERTINIT' --l_action_type
                                        ,'N' -- l_need_to_commit
                                        ,L_BASE_RULE_ID
                                        ,L_RULE_VERSION_NBR
                                        ,L_PAUSE_TIMESTAMP
                                        ,L_PAUSE_ELAPSED_TIME
                                        ,NULL -- ,l_strategy_id
                        );
                    
                         P_ALERT_ID := L_ALERT_ID;
                    
                    ELSIF ( L_ALERT_STATUS = 'CLOSED' ) THEN
                                --
                                -- update fcm_alert set status = 'ACTIVE'
                                --
                                IF (L_SUSPEND = 'true') THEN
                                    INSERT INTO FCM_ALERT_SUSPEND (CONTACT_VALUE,CONTACT_TYPE,TENANT_MULTI_ORG_ID,ALERT_ID,SUSPEND_USER,SUSPEND_TIMESTAMP)
                                    SELECT L_CONTACT_VALUE,L_CONTACT_TYPE,TENANT_MULTI_ORG_ID,ALERT_ID,I_USER_NAME,SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)
                                    FROM FCM_ALERT
                                    WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND
                                                        WHERE CONTACT_VALUE = L_CONTACT_VALUE
                                                        AND CONTACT_TYPE = L_CONTACT_TYPE
                                                        and TENANT_MULTI_ORG_ID = L_TENANT_MULTI_ORG_ID
/* commented out for race condition*/                                                        
--                                                          AND SUSPEND_USER <> I_USER_NAME
--                                                          AND ALERT_ID <> L_ALERT_ID
                                                          )
                                        AND FCM_ALERT.CONTACT_VALUE IS NOT NULL
										AND ALERT_ID = L_ALERT_ID
                                        AND ALERT_STATUS='CLOSED';
                                    L_ROW_COUNT := SQL%ROWCOUNT;
                                     


                                    If(l_row_count = 0 AND L_CONTACT_VALUE IS NOT NULL) then
                                        update fcm_alert_suspend
                                          set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
                                        where contact_value = l_contact_value
                                            and contact_type = l_contact_type 
                                            and TENANT_MULTI_ORG_ID = L_TENANT_MULTI_ORG_ID
                                            and suspend_user = i_user_name;
            
                                        L_ROW_COUNT := SQL%ROWCOUNT;
                                    end if;    
                                    IF(L_ROW_COUNT <> 1 AND L_CONTACT_VALUE IS NOT NULL) THEN
                                        L_ERROR := 1;
                                        L_ALERT_ID := -1;
                                    END IF;
                                END IF;        
                                IF (L_ERROR <> 1) THEN
                                UPDATE FCM_ALERT SET
                                        QUEUE_ID = I_QUEUE_ID
                                        ,ALERT_VERSION_NBR = ALERT_VERSION_NBR + 1
                                        ,MULTI_ORG_ID = L_MULTI_ORG_ID
                                        ,CONTACT_VALUE = I_CONTACT_VALUE
                                        ,CONTACT_TYPE = I_CONTACT_TYPE
                                        ,PARENT_ALERT_VALUE = I_PARENT_ALERT_VALUE
                                        ,PARENT_ALERT_TYPE = I_PARENT_ALERT_TYPE                                        
                                        ,ALERT_TYPE =  L_ALERT_TYPE
                                        ,ALERT_STATUS = 'CHECKOUT'
                                        ,GMT_OFFSET = L_GMT_OFFSET
                                        ,FRAUD_SCORE = L_FRAUD_SCORE
                                        ,FRAUD_SCORE_REASON_CODE_1 = L_FRAUD_SCORE_REASON_CODE_1
                                        ,INIT_FRAUD_SCORE  = L_FRAUD_SCORE
                                        ,INIT_FRAUD_SCORE_REASON_CODE_1  = L_FRAUD_SCORE_REASON_CODE_1
                                        ,HWM_FRAUD_SCORE  = L_FRAUD_SCORE
                                        ,HWM_FRAUD_SCORE_REASON_CODE_1  = L_FRAUD_SCORE_REASON_CODE_1
                                        ,ALERT_SERVICE_CNT = 1
                                        ,ALERT_ORIGIN = 'INBOUND'
                                        ,INIT_TRANSACTION_AMT = L_LST_TRANSACTION_AMT
                                        ,INIT_TRANSACTION_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP
                                        ,LST_TRANSACTION_AMT = L_LST_TRANSACTION_AMT
                                        ,LST_TRANSACTION_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,USER_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,SYSTEM_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,NEXT_SERVICE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,QUEUE_ENTRY_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,CHECKOUT_USER = I_USER_NAME
                                        ,CHECKOUT_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,CREATE_USER = I_USER_NAME
                                        ,CREATE_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,LSTUPDT_USER = I_USER_NAME
                                        ,LSTUPDT_TIMESTAMP   = L_CURRENT_UTC_TIMESTAMP
                                        ,LSTUPDT_NBR = 0
                                        ,LOGICAL_DEL_IND = '0'
                                    WHERE ALERT_ID = L_ALERT_ID;
                    
                                 GET_ALERT_ROW
                                        ( L_ALERT_ID
                                        ,L_ALRT_VER_NBR
                                        ,L_ALERT_VALUE
                                        ,L_PARENT_ALERT_VALUE
                                        ,L_PARENT_ALERT_TYPE
                                        ,L_CONTACT_VALUE
                                        ,L_CONTACT_TYPE 
                                        ,L_TENANT_MULTI_ORG_ID   
                                        ,L_QUEUE_ID
                                        ,L_MULTI_ORG_ID
                                        ,L_ALERT_TYPE
                                        ,L_RESULT_ID
                                        ,L_ALERT_STATUS
                                        ,L_GMT_OFFSET
                                        ,L_FRAUD_TYPE_CODE
                                        ,L_FRAUD_SCORE
                                        ,L_FRAUD_SCORE_REASON_CODE_1
                                        ,L_FRAUD_SCORE_REASON_CODE_2
                                        ,L_FRAUD_SCORE_REASON_CODE_3
                                        ,L_INIT_FRAUD_SCORE
                                        ,L_INIT_FRD_SCORE_REASON_CODE_1
                                        ,L_INIT_FRD_SCORE_REASON_CODE_2
                                        ,L_INIT_FRD_SCORE_REASON_CODE_3
                                        ,L_HWM_FRAUD_SCORE
                                        ,L_HWM_FRD_SCORE_REASON_CODE_1
                                        ,L_HWM_FRD_SCORE_REASON_CODE_2
                                        ,L_HWM_FRD_SCORE_REASON_CODE_3
                                        ,L_ALERT_SERVICE_CNT
                                        ,L_ALERT_ORIGIN
                                        ,L_LST_TRANSACTION_AMT
                                        ,L_LST_TRANSACTION_TIMESTAMP
                                        ,L_USER_SERVICE_TIMESTAMP
                                        ,L_SYSTEM_SERVICE_TIMESTAMP
                                        ,L_NEXT_SERVICE_TIMESTAMP
                                        ,L_QUEUE_ENTRY_TIMESTAMP
                                        ,L_FINAL_DISPOSITION_TIMESTAMP
                                        ,L_CHECKOUT_USER
                                        ,L_CHECKOUT_TIMESTAMP
                                        ,L_USER_NAME
                                        ,L_CREATE_TIMESTAMP
                                        ,L_USER_NAME
                                        ,L_LSTUPDT_TIMESTAMP
                                        ,L_LSTUPDT_NBR
                                        ,L_BASE_RULE_ID
                                        ,L_RULE_VERSION_NBR
                                        ,L_PAUSE_TIMESTAMP
                                        ,L_PAUSE_ELAPSED_TIME
                                        );
   
		                    IF (L_SUSPEND = 'true') THEN
		                    	INSERT INTO FCM_ALERT_ACTION (
		                    				ALERT_ACTION_ID,
		                    				ALERT_ID ,
		                                    ALERT_VERSION_NBR ,
		                                    ACTION_TIMESTAMP ,
		                                    CONTACT_VALUE ,
		                                    CONTACT_TYPE ,
		                                    TENANT_MULTI_ORG_ID,
		                                    ALERT_VALUE ,
		                                    ALERT_TYPE ,
		                                    MULTI_ORG_ID,
		                                    ALERT_STATUS ,
		                                    GMT_OFFSET ,
		                                    ALERT_SERVICE_CNT ,
		                                    ALERT_ORIGIN ,
		                                    CREATE_USER ,
		                                    CREATE_TIMESTAMP )
		                            VALUES( ALERT_ACTION_SEQ.nextval,
		                            		L_ALERT_ID ,
		                                    L_ALRT_VER_NBR ,
		                                    L_CURRENT_UTC_TIMESTAMP ,
		                                    L_CONTACT_VALUE ,
		                                    L_CONTACT_TYPE ,
		                                    l_TENANT_MULTI_ORG_ID,
		                                    l_ALERT_VALUE ,
		                                    l_ALERT_TYPE ,
		                                    L_MULTI_ORG_ID,
		                                    'SUSPEND' ,
		                                    l_GMT_OFFSET ,
		                                    L_ALERT_SERVICE_CNT ,
		                                    l_ALERT_ORIGIN ,
		                                    I_USER_NAME ,
		                                    l_CREATE_TIMESTAMP  );
                				END IF;
                				
                                 SAVE_ALERT_ACTION
                                        ( L_ALERT_ID
                                            ,L_ALRT_VER_NBR
                                            ,L_ALERT_VALUE
                                            ,L_PARENT_ALERT_VALUE
                                            ,L_PARENT_ALERT_TYPE                                            
                                            ,L_CONTACT_VALUE 
                                            ,L_CONTACT_TYPE 
                                            ,L_TENANT_MULTI_ORG_ID
                                            ,L_QUEUE_ID
                                            ,L_MULTI_ORG_ID
                                            ,L_ALERT_TYPE
                                            ,L_RESULT_ID
                                            ,L_ALERT_STATUS
                                            ,L_GMT_OFFSET
                                            ,L_FRAUD_TYPE_CODE
                                            ,L_FRAUD_SCORE
                                            ,L_FRAUD_SCORE_REASON_CODE_1
                                            ,L_FRAUD_SCORE_REASON_CODE_2
                                            ,L_FRAUD_SCORE_REASON_CODE_3
                                            ,L_INIT_FRAUD_SCORE
                                            ,L_INIT_FRD_SCORE_REASON_CODE_1
                                            ,L_INIT_FRD_SCORE_REASON_CODE_2
                                            ,L_INIT_FRD_SCORE_REASON_CODE_3
                                            ,L_HWM_FRAUD_SCORE
                                            ,L_HWM_FRD_SCORE_REASON_CODE_1
                                            ,L_HWM_FRD_SCORE_REASON_CODE_2
                                            ,L_HWM_FRD_SCORE_REASON_CODE_3
                                            ,L_ALERT_SERVICE_CNT
                                            ,L_ALERT_ORIGIN
                                            ,L_LST_TRANSACTION_AMT
                                            ,L_LST_TRANSACTION_TIMESTAMP
                                            ,L_USER_SERVICE_TIMESTAMP
                                            ,L_SYSTEM_SERVICE_TIMESTAMP
                                            ,L_NEXT_SERVICE_TIMESTAMP
                                            ,L_QUEUE_ENTRY_TIMESTAMP
                                            ,L_FINAL_DISPOSITION_TIMESTAMP
                                            ,L_CHECKOUT_USER
                                            ,L_CHECKOUT_TIMESTAMP
                                            ,I_USER_NAME
                                            ,L_CREATE_TIMESTAMP
                                            ,I_USER_NAME
                                            ,L_LSTUPDT_TIMESTAMP
                                            ,L_LSTUPDT_NBR
                                        --    ,NULL --,l_pend_reason_code
                                            ,NULL --,l_reroute_queue_id
                                            ,'INB' -- ,l_service_method_code
                                            ,'INITCLOSED' --l_action_type    (need to specify manual?)
                                            ,'N' -- l_need_to_commit
                                            ,L_BASE_RULE_ID
                                            ,L_RULE_VERSION_NBR
                                            ,L_PAUSE_TIMESTAMP
                                            ,L_PAUSE_ELAPSED_TIME
                                            ,NULL --,l_strategy_id
                                );
                                END IF;
                         P_ALERT_ID := L_ALERT_ID;
                    
                    END IF;
            
            ELSE
            
            
                    -- normal logic flow, ie an alert doesn't already exist for this card
                    -- step 2. insert alert
                    IF (L_SUSPEND = 'true') THEN
                        INSERT INTO FCM_ALERT_SUSPEND (CONTACT_VALUE,CONTACT_TYPE,TENANT_MULTI_ORG_ID,SUSPEND_USER,SUSPEND_TIMESTAMP)
                        SELECT I_CONTACT_VALUE,I_CONTACT_TYPE,L_TENANT_MULTI_ORG_ID,I_USER_NAME,SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)
                        FROM DUAL
                        WHERE NOT EXISTS (SELECT 1 FROM FCM_ALERT_SUSPEND
                                            WHERE CONTACT_VALUE = I_CONTACT_VALUE
                                            AND CONTACT_TYPE = I_CONTACT_TYPE
                                            AND TENANT_MULTI_ORG_ID = L_TENANT_MULTI_ORG_ID
                                            AND I_CONTACT_VALUE IS NOT NULL
                                        );
                            L_ROW_COUNT := SQL%ROWCOUNT;
                             If(l_row_count = 0 AND L_CONTACT_VALUE IS NOT NULL) then
                                        update fcm_alert_suspend
                                          set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT+1
                                        where contact_value = i_contact_value
                                            and contact_type = i_contact_type 
                                            and TENANT_multi_org_id = l_TENANT_MULTI_ORG_ID
                                            and suspend_user = i_user_name;
            
                                        L_ROW_COUNT := SQL%ROWCOUNT;
                            end if;        
                        IF (l_row_count <> 1 AND L_CONTACT_VALUE IS NOT NULL)THEN
                              L_ERROR := 1;
                              P_ALERT_ID := -1;
                         END IF;
                    END IF;
                    IF (L_ERROR <> 1) THEN
                        Select ALERT_SEQ.nextval into P_ALERT_ID from DUAL;

                    INSERT INTO FCM_ALERT(
                        ALERT_ID,
                        ALERT_VALUE,
                        PARENT_ALERT_VALUE,
                        PARENT_ALERT_TYPE,
                        CONTACT_VALUE,
                        CONTACT_TYPE,
                        TENANT_MULTI_ORG_ID,
                        QUEUE_ID,
                        MULTI_ORG_ID,
                        ALERT_TYPE,
                        ALERT_STATUS,
                        GMT_OFFSET,
                        ALERT_SERVICE_CNT,
                        ALERT_ORIGIN,
                        USER_SERVICE_TIMESTAMP,
                        SYSTEM_SERVICE_TIMESTAMP,
                        NEXT_SERVICE_TIMESTAMP,
                        QUEUE_ENTRY_TIMESTAMP,
                        CHECKOUT_USER,
                        CHECKOUT_TIMESTAMP,
                        CREATE_USER,
                        CREATE_TIMESTAMP,
                        LSTUPDT_USER,
                        LSTUPDT_TIMESTAMP,
                        LSTUPDT_NBR,
                        LOGICAL_DEL_IND
                                    )
                    VALUES(
                        P_ALERT_ID,
                        I_ALERT_VALUE,
                        I_PARENT_ALERT_VALUE,
                        I_PARENT_ALERT_TYPE,
                        I_CONTACT_VALUE,
                        I_CONTACT_TYPE,
                        l_TENANT_MULTI_ORG_ID,
                        I_QUEUE_ID,
                        L_MULTI_ORG_ID,
                        L_ALERT_TYPE,
                        'CHECKOUT',
                        L_GMT_OFFSET,
                        1,
                        'INBOUND',
                        L_CURRENT_UTC_TIMESTAMP,
                        L_CURRENT_UTC_TIMESTAMP,
                        L_CURRENT_UTC_TIMESTAMP,
                        L_CURRENT_UTC_TIMESTAMP,
                        I_USER_NAME,
                        L_CURRENT_UTC_TIMESTAMP,
                        I_USER_NAME,
                        L_CURRENT_UTC_TIMESTAMP,
                        I_USER_NAME,
                        L_CURRENT_UTC_TIMESTAMP,
                        0,
                        '0'
                    );
                    
                    --VALUES IDENTITY_VAL_LOCAL()    INTO  P_ALERT_ID;
                    IF (L_SUSPEND = 'true') THEN
                         INSERT INTO FCM_ALERT_ACTION (
                                    ALERT_ACTION_ID ,
                    				ALERT_ID ,
                                    ALERT_VERSION_NBR ,
                                    ACTION_TIMESTAMP ,
                                    CONTACT_VALUE ,
                                    CONTACT_TYPE ,
                                    TENANT_MULTI_ORG_ID,
                                    ALERT_VALUE ,
                                    ALERT_TYPE ,
                                    MULTI_ORG_ID,
                                    ALERT_STATUS ,
                                    GMT_OFFSET ,
                                    ALERT_SERVICE_CNT ,
                                    ALERT_ORIGIN ,
                                    CREATE_USER ,
                                    CREATE_TIMESTAMP
									)
                            VALUES( ALERT_ACTION_SEQ.nextval,
									P_ALERT_ID ,
                                    0 ,
                                    L_CURRENT_UTC_TIMESTAMP ,
                                    I_CONTACT_VALUE ,
                                    I_CONTACT_TYPE,
                                    L_TENANT_MULTI_ORG_ID,
                                    I_ALERT_VALUE ,
                                    l_ALERT_TYPE ,
                                    L_MULTI_ORG_ID,
                                    'SUSPEND' ,
                                    l_GMT_OFFSET ,
                                    1,
                                    'INBOUND',
                                    I_USER_NAME ,
                                    L_CURRENT_UTC_TIMESTAMP  );
                    END IF;                    

                    -- step 2a. get alert
                    
                    SELECT
                        ALERT_ID,
                        ALERT_VERSION_NBR,
                        ALERT_VALUE,
                        PARENT_ALERT_VALUE,
                        PARENT_ALERT_TYPE,
                        CONTACT_VALUE,
                        CONTACT_TYPE,
                        TENANT_MULTI_ORG_ID,
                        QUEUE_ID,
                        MULTI_ORG_ID,
                        ALERT_TYPE,
                        RESULT_ID,
                        ALERT_STATUS,
                        GMT_OFFSET,
                        FRAUD_TYPE_CODE,
                        PEND_REASON_CODE,
                        FRAUD_SCORE,
                        FRAUD_SCORE_REASON_CODE_1,
                        FRAUD_SCORE_REASON_CODE_2,
                        FRAUD_SCORE_REASON_CODE_3,
                        INIT_FRAUD_SCORE,
                        INIT_FRAUD_SCORE_REASON_CODE_1,
                        INIT_FRAUD_SCORE_REASON_CODE_2,
                        INIT_FRAUD_SCORE_REASON_CODE_3,
                        HWM_FRAUD_SCORE,
                        HWM_FRAUD_SCORE_REASON_CODE_1,
                        HWM_FRAUD_SCORE_REASON_CODE_2,
                        HWM_FRAUD_SCORE_REASON_CODE_3,
                        ALERT_SERVICE_CNT,
                        ALERT_ORIGIN,
                        LST_TRANSACTION_AMT,
                        LST_TRANSACTION_TIMESTAMP,
                        USER_SERVICE_TIMESTAMP,
                        SYSTEM_SERVICE_TIMESTAMP,
                        NEXT_SERVICE_TIMESTAMP,
                        QUEUE_ENTRY_TIMESTAMP,
                        FINAL_DISPOSITION_TIMESTAMP,
                        CHECKOUT_USER,
                        CHECKOUT_TIMESTAMP,
                        CREATE_USER,
                        CREATE_TIMESTAMP,
                        LSTUPDT_USER,
                        LSTUPDT_TIMESTAMP,
                        LSTUPDT_NBR,
                        LOGICAL_DEL_IND,
                        BASE_RULE_ID,
                        RULE_VERSION_NBR
                    INTO
                        L_ALERT_ID,
                        L_ALRT_VER_NBR,
                        L_ALERT_VALUE,
                        L_PARENT_ALERT_VALUE,
                        L_PARENT_ALERT_TYPE,
                        L_CONTACT_VALUE,
                        L_CONTACT_TYPE,
                        L_TENANT_MULTI_ORG_ID,
                        L_QUEUE_ID,
                        L_MULTI_ORG_ID,
                        L_ALERT_TYPE,
                        L_RESULT_ID ,
                        L_ALERT_STATUS,
                        L_GMT_OFFSET,
                        L_FRAUD_TYPE_CODE ,
                        L_PEND_REASON_CODE,
                        L_FRAUD_SCORE ,
                        L_FRAUD_SCORE_REASON_CODE_1 ,
                        L_FRAUD_SCORE_REASON_CODE_2 ,
                        L_FRAUD_SCORE_REASON_CODE_3 ,
                        L_INIT_FRAUD_SCORE,
                        L_INIT_FRD_SCORE_REASON_CODE_1,
                        L_INIT_FRD_SCORE_REASON_CODE_2,
                        L_INIT_FRD_SCORE_REASON_CODE_3,
                        L_HWM_FRAUD_SCORE ,
                        L_HWM_FRD_SCORE_REASON_CODE_1 ,
                        L_HWM_FRD_SCORE_REASON_CODE_2 ,
                        L_HWM_FRD_SCORE_REASON_CODE_3 ,
                        L_ALERT_SERVICE_CNT ,
                        L_ALERT_ORIGIN,
                        L_LST_TRANSACTION_AMT ,
                        L_LST_TRANSACTION_TIMESTAMP ,
                        L_USER_SERVICE_TIMESTAMP,
                        L_SYSTEM_SERVICE_TIMESTAMP,
                        L_NEXT_SERVICE_TIMESTAMP,
                        L_QUEUE_ENTRY_TIMESTAMP ,
                        L_FINAL_DISPOSITION_TIMESTAMP ,
                        L_CHECKOUT_USER ,
                        L_CHECKOUT_TIMESTAMP,
                        L_CREATE_USER ,
                        L_CREATE_TIMESTAMP,
                        L_LSTUPDT_USER,
                        L_LSTUPDT_TIMESTAMP ,
                        L_LSTUPDT_NBR ,
                        L_LOGICAL_DEL_IND,
                        L_BASE_RULE_ID,
                        L_RULE_VERSION_NBR
                    FROM FCM_ALERT
                    WHERE
                        ALERT_ID = P_ALERT_ID;
                    
                     SAVE_ALERT_ACTION
                                        ( L_ALERT_ID
                                            ,L_ALRT_VER_NBR
                                            ,L_ALERT_VALUE
                                            ,L_PARENT_ALERT_VALUE
                                            ,L_PARENT_ALERT_TYPE
                                            ,L_CONTACT_VALUE
                                            ,L_CONTACT_TYPE
                                            ,L_TENANT_MULTI_ORG_ID
                                            ,L_QUEUE_ID
                                            ,L_MULTI_ORG_ID
                                            ,L_ALERT_TYPE
                                            ,L_RESULT_ID
                                            ,L_ALERT_STATUS
                                            ,L_GMT_OFFSET
                                            ,L_FRAUD_TYPE_CODE
                                            ,L_FRAUD_SCORE
                                            ,L_FRAUD_SCORE_REASON_CODE_1
                                            ,L_FRAUD_SCORE_REASON_CODE_2
                                            ,L_FRAUD_SCORE_REASON_CODE_3
                                            ,L_INIT_FRAUD_SCORE
                                            ,L_INIT_FRD_SCORE_REASON_CODE_1
                                            ,L_INIT_FRD_SCORE_REASON_CODE_2
                                            ,L_INIT_FRD_SCORE_REASON_CODE_3
                                            ,L_HWM_FRAUD_SCORE
                                            ,L_HWM_FRD_SCORE_REASON_CODE_1
                                            ,L_HWM_FRD_SCORE_REASON_CODE_2
                                            ,L_HWM_FRD_SCORE_REASON_CODE_3
                                            ,L_ALERT_SERVICE_CNT
                                            ,L_ALERT_ORIGIN
                                            ,L_LST_TRANSACTION_AMT
                                            ,L_LST_TRANSACTION_TIMESTAMP
                                            ,L_USER_SERVICE_TIMESTAMP
                                            ,L_SYSTEM_SERVICE_TIMESTAMP
                                            ,L_NEXT_SERVICE_TIMESTAMP
                                            ,L_QUEUE_ENTRY_TIMESTAMP
                                            ,L_FINAL_DISPOSITION_TIMESTAMP
                                            ,L_CHECKOUT_USER
                                            ,L_CHECKOUT_TIMESTAMP
                                            ,I_USER_NAME
                                            ,L_CREATE_TIMESTAMP
                                            ,I_USER_NAME
                                            ,L_LSTUPDT_TIMESTAMP
                                            ,L_LSTUPDT_NBR
                                            --,NULL --,l_pend_reason_code
                                            ,NULL --,l_reroute_queue_id
                                            ,'INB' -- ,l_service_method_code
                                            ,'ALERTINIT' --l_action_type
                                            ,'N' -- l_need_to_commit
                                            ,L_BASE_RULE_ID
                                            ,L_RULE_VERSION_NBR
                                            ,L_PAUSE_TIMESTAMP
                                            ,L_PAUSE_ELAPSED_TIME
                                            ,NULL -- ,l_strategy_id
                        );
            END IF;
            END IF;
            
            -- step 4. commit if y
            IF (I_NEED_TO_COMMIT = 'Y') THEN
                    COMMIT;
                     GET_ALERT(P_ALERT_ID, I_USER_NAME,o_getalertcur);
            END IF;

ELSE
     GET_ALERT(-1, I_USER_NAME,o_getalertcur);
END IF;
--
END CR_MAN_ALERT;
/
