CREATE OR REPLACE PROCEDURE INSERT_IVR_ALERT_ACTION (
     I_ALERT_ID    IN NUMBER,
     I_MEMO_TEXT    IN VARCHAR2,
     I_INVALID_CONTACT_DETAILS_IND    IN VARCHAR2,
     I_CONTACT_HOME_PHONE_IND    IN VARCHAR2,
     I_CONTACT_WORK_PHONE_IND    IN VARCHAR2,
     I_CONTACT_MOBILE_PHONE_IND    IN VARCHAR2,
     I_USER_NAME    IN VARCHAR2,
     I_USER_ID    IN NUMBER,
     I_NEED_TO_COMMIT    IN VARCHAR2,
     I_FULFILL_SENT_CODE    IN VARCHAR2,
     O_ALERT_AVAILABLE    OUT VARCHAR2,
     O_SQLCODE    OUT NUMBER,
     O_SQLSTATE    OUT VARCHAR2 )

-----------------------------------------------------------------------------
-- File:  INSERT_IVR_ALERT_ACTION
--
--
-- Description:
--
--
--
-- Called By:
--     n/a
--
-- Calls:
--      GET_ALERT_ROW
--      SAVE_ALERT_FULFIL
--
-- Notes:
--        Save an IVR alert action record
--      Called by the webservice, so we don't validate the users access to the alert
--
-- Usage:
--        Called from the web Service
--
--
-- History:
--     2012/07/23    leushe      Ported from DB2
--     2013/03/12    leushe      Change support to alert/entity values of 100 bytes
--     2014/10/05   leushe  Changed the servicable check to be coalesce(l_result_id,0)  < 7 and NEXT_SERVICE_TIMESTAMP < l_now
--
------------------------------------------------------------------------
IS


    l_exists number default 0;
    l_notexists number default 0;
    l_alert_status VARCHAR2(10);
    l_TENANT_multi_org_id integer;
    l_queue_id NUMBER;
    L_CONTACT_TYPE VARCHAR2(1) ;
    l_contact_value varchar2(100);
    l_result_id  integer;
    l_action_type VARCHAR2(20) DEFAULT 'IVR ACTION';
    l_alert_action_id NUMBER;
    l_now TIMESTAMP;
    l_next_service_timestamp TIMESTAMP;
    
BEGIN
        l_now := SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);

      select alert_Status  
            ,tenant_multi_org_id  
            ,queue_id
            ,result_id
            ,next_service_timestamp
            ,l_contact_value
            ,contact_type into l_alert_status, l_tenant_multi_org_id,l_queue_id,l_result_id,l_next_service_timestamp,l_contact_value,l_contact_type
      from 
        FCM_ALERT where ALERT_ID=I_ALERT_ID;
      
      Select ALERT_ACTION_SEQ.nextval into L_ALERT_ACTION_ID from DUAL;

      -- Insert new alert action
      INSERT INTO FCM_ALERT_ACTION(
                    ALERT_ACTION_ID,
                    ALERT_ID,
                    ALERT_VERSION_NBR,
                    ACTION_TIMESTAMP,
                    PARENT_ALERT_VALUE,
                    PARENT_ALERT_TYPE,
                    ALERT_VALUE,
                    CONTACT_VALUE,
                    CONTACT_TYPE,
                    TENANT_MULTI_ORG_ID,
                    QUEUE_ID,
                    MULTI_ORG_ID,
                    ALERT_TYPE,
                    RESULT_ID,
                    ALERT_STATUS,
                    GMT_OFFSET,
                    FRAUD_TYPE_CODE,
                    FRAUD_SCORE,
                    FRAUD_SCORE_REASON_CODE_1,
                    FRAUD_SCORE_REASON_CODE_2,
                    FRAUD_SCORE_REASON_CODE_3,
                    INIT_FRAUD_SCORE,
                    INIT_FRAUD_SCORE_REASON_CODE_1,
                    INIT_FRAUD_SCORE_REASON_CODE_2,
                    INIT_FRAUD_SCORE_REASON_CODE_3,
                    HWM_FRAUD_SCORE,
                    HWM_FRAUD_SCORE_REASON_CODE_1,
                    HWM_FRAUD_SCORE_REASON_CODE_2,
                    HWM_FRAUD_SCORE_REASON_CODE_3,
                    ALERT_SERVICE_CNT,
                    ALERT_ORIGIN,
                    LST_TRANSACTION_AMT,
                    LST_TRANSACTION_TIMESTAMP,
                    USER_SERVICE_TIMESTAMP,
                    SYSTEM_SERVICE_TIMESTAMP,
                    NEXT_SERVICE_TIMESTAMP,
                    QUEUE_ENTRY_TIMESTAMP,
                    FINAL_DISPOSITION_TIMESTAMP,
                    CHECKOUT_USER,
                    CHECKOUT_TIMESTAMP,
                    CREATE_USER,
                    CREATE_TIMESTAMP,
                    LSTUPDT_USER,
                    LSTUPDT_TIMESTAMP,
                    LSTUPDT_NBR,
                    CONTACT_HOME_PHONE_IND,
                    CONTACT_MOBILE_PHONE_IND,
                    CONTACT_WORK_PHONE_IND,
                    ACTION_TYPE,
                    BASE_RULE_ID,
                    RULE_VERSION_NBR,
                    PAUSE_TIMESTAMP,
                    PAUSE_ELAPSED_TIME,
                    STRATEGY_ID
                    )
                    SELECT  L_ALERT_ACTION_ID,
                    ALERT_ID,
                    ALERT_VERSION_NBR,
                    l_now,
                    PARENT_ALERT_VALUE,
                    PARENT_ALERT_TYPE,
                    ALERT_VALUE,
                    CONTACT_VALUE,
                    CONTACT_TYPE,
                    TENANT_MULTI_ORG_ID,
                    a.QUEUE_ID,
                    MULTI_ORG_ID,
                    ALERT_TYPE,
                    0,
                    ALERT_STATUS,
                    GMT_OFFSET,
                    FRAUD_TYPE_CODE,
                    FRAUD_SCORE,
                    FRAUD_SCORE_REASON_CODE_1,
                    FRAUD_SCORE_REASON_CODE_2,
                    FRAUD_SCORE_REASON_CODE_3,
                    INIT_FRAUD_SCORE,
                    INIT_FRAUD_SCORE_REASON_CODE_1,
                    INIT_FRAUD_SCORE_REASON_CODE_2,
                    INIT_FRAUD_SCORE_REASON_CODE_3,
                    HWM_FRAUD_SCORE,
                    HWM_FRAUD_SCORE_REASON_CODE_1,
                    HWM_FRAUD_SCORE_REASON_CODE_2,
                    HWM_FRAUD_SCORE_REASON_CODE_3,
                    ALERT_SERVICE_CNT,
                    ALERT_ORIGIN,
                    LST_TRANSACTION_AMT,
                    LST_TRANSACTION_TIMESTAMP,
                    USER_SERVICE_TIMESTAMP,
                    SYSTEM_SERVICE_TIMESTAMP,
                    NEXT_SERVICE_TIMESTAMP,
                    QUEUE_ENTRY_TIMESTAMP,
                    FINAL_DISPOSITION_TIMESTAMP,
                    CHECKOUT_USER,
                    CHECKOUT_TIMESTAMP,
                    i_user_name ,
                    l_now,
                    i_USER_NAME,
                    l_now,
                    a.LSTUPDT_NBR,
                    i_contact_home_phone_ind,
                    i_contact_mobile_phone_ind,
                    i_contact_work_phone_ind,                    
                    l_action_type,
                    BASE_RULE_ID,
                    RULE_VERSION_NBR,
                    PAUSE_TIMESTAMP,
                    PAUSE_ELAPSED_TIME,
                    STRATEGY_ID
                    FROM FCM_ALERT a, FCM_QUEUE q
                    where a.queue_id=q.queue_id
                    and a.alert_id = I_ALERT_ID;
                    
                 


     --l_alert_action_id := identity_val_local();

     SAVE_ALERT_FULFIL(
               l_alert_action_id,
               '0', -- i_SENT_LETTER_IND,
               '0', -- i_SENT_EMAIL_IND,
               '0',  -- i_SENT_SMS_IND,
               i_MEMO_TEXT
               ,i_USER_NAME
               ,'0'
               ,i_INVALID_CONTACT_DETAILS_IND
               ,i_fulfill_sent_code
               ,NULL  --i_FULFIll_XML_MSG
               ,o_SQLCODE
               ,o_SQLSTATE
               );

    IF i_need_to_commit = 'Y' THEN
        COMMIT;
    END IF;

     o_sqlcode := 0;
     o_sqlstate := 'SUCES';



    select count(1) into l_notexists
                from fcm_alert_suspend fas
                where l_contact_value = fas.contact_value 
                    and l_contact_type = fas.contact_type
                    and l_TENANT_multi_org_id = fas.TENANT_multi_org_id
                    and fas.suspend_user !=i_user_name
                     and rownum = 1;

 select count(1) into l_exists
                        from fcm_strategy_user su, fcm_queue q, fcm_strategy s
                                where q.queue_id = l_queue_id and
                                      su.strategy_id = q.strategy_id and
                                      s.strategy_id = q.strategy_id and
                                      s.strategy_status = 'ACTIVE' and
                                      q.queue_status in ('ACTIVE', 'QUIESCE') and
                                      su.user_id = i_user_id and
                      rownum = 1;

 IF (l_notexists = 0 AND l_exists = 1            AND
                l_alert_status = 'ACTIVE'
            AND
                coalesce(l_result_id,0) < 7
            AND 
                l_next_service_timestamp < l_now) THEN

         o_alert_available := '1';
    ELSE
         o_alert_available := '0';
    END IF;
    update fcm_ivr_alert set update_nbr = update_nbr + 1 where alert_id = i_alert_id;
    
    
  EXCEPTION
  WHEN NO_DATA_FOUND THEN
    BEGIN
     O_ALERT_AVAILABLE  := '0';     
     O_SQLCODE    :=0;
     O_SQLSTATE   :='75001';
    END;

END INSERT_IVR_ALERT_ACTION;
/
show errors;