
  CREATE OR REPLACE PROCEDURE UNCHECKOUT_SPECIFIC_ALERT (
    I_ALERT_ID IN NUMBER,
    I_USER_NAME IN VARCHAR2
)

-----------------------------------------------------------------------------
-- File:   uncheckout_alert.sql
--
--
-- Description:  procedure unchecks out a specific alert. It deletes the alert from the
--               FCM_ALERT_SUSPEND table, if existing
--
--
-- Called By: uncheckout_alert
--
--
-- Calls:
--     None
--
-- Notes:
--
-- Usage:
--		I_ALERT_ID alert_id to be unchecked out
--
--
-- History:
--     2012/07/23    leushe	Ported from DB2
--     2013/03/12    leushe Change support to alert/entity values of 100 bytes
--     2014/03/05    leushe Changed the query to check for alert_Suspend_cnt to use coalesce(max(alert_checkout_cnt),0) so there is always a result
--                          and fix bug to set the FCM_ALERT_ACTION.TENANT_MULTI_ORG_ID
--     2015/06/01   leushe  Moved Update to FCM_ALERT before Insert to FCM_ALERT_ACTION so LSTUPDT fields are correct in ALERT_ACTION
--
--
------------------------------------------------------------------------

IS

	L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
   L_CONTACT_VALUE  varchar2(100);
   L_CONTACT_TYPE  varchar2(1);
   L_TENANT_MULTI_ORG_ID INTEGER;
	L_CNTR SMALLINT;

BEGIN
--Set the current timestamp once
			 L_CURRENT_UTC_TIMESTAMP := SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);

			 UPDATE FCM_ALERT
				SET ALERT_STATUS = 'ACTIVE',
				  CHECKOUT_USER = NULL,
				  CHECKOUT_TIMESTAMP = NULL,
				  LSTUPDT_USER = I_USER_NAME,
				  LSTUPDT_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP,
				  LSTUPDT_NBR = LSTUPDT_NBR + 1
				WHERE ALERT_ID = I_ALERT_ID;

				INSERT INTO FCM_ALERT_ACTION
                          (ALERT_ACTION_ID,
                           ACTION_TIMESTAMP,
                           ACTION_TYPE,
                           ALERT_ID,
                           ALERT_VERSION_NBR,
                           PARENT_ALERT_VALUE,
                           PARENT_ALERT_TYPE,
                           ALERT_VALUE,
                           CONTACT_VALUE,
                           CONTACT_TYPE,
                           TENANT_MULTI_ORG_ID,                           
                           ALERT_TYPE,
                           QUEUE_ID,
                           MULTI_ORG_ID,
                           RESULT_ID,
                           ALERT_STATUS,
                           GMT_OFFSET,
                           FRAUD_TYPE_CODE,
                           FRAUD_SCORE,
                           FRAUD_SCORE_REASON_CODE_1,
                           FRAUD_SCORE_REASON_CODE_2,
                           FRAUD_SCORE_REASON_CODE_3,
                           INIT_FRAUD_SCORE,
                           INIT_FRAUD_SCORE_REASON_CODE_1,
                           INIT_FRAUD_SCORE_REASON_CODE_2,
                           INIT_FRAUD_SCORE_REASON_CODE_3,
                           HWM_FRAUD_SCORE,
                           HWM_FRAUD_SCORE_REASON_CODE_1,
                           HWM_FRAUD_SCORE_REASON_CODE_2,
                           HWM_FRAUD_SCORE_REASON_CODE_3,
                           ALERT_SERVICE_CNT,
                           ALERT_ORIGIN,
                           INIT_TRANSACTION_AMT,
                           INIT_TRANSACTION_TIMESTAMP,
                           LST_TRANSACTION_AMT,
                           LST_TRANSACTION_TIMESTAMP,
                           USER_SERVICE_TIMESTAMP,
                           SYSTEM_SERVICE_TIMESTAMP,
                           NEXT_SERVICE_TIMESTAMP,
                           QUEUE_ENTRY_TIMESTAMP,
                           FINAL_DISPOSITION_TIMESTAMP,
                           CHECKOUT_USER,
                           CHECKOUT_TIMESTAMP,
                           CREATE_USER,
                           CREATE_TIMESTAMP,
                           LSTUPDT_USER,
                           LSTUPDT_TIMESTAMP,
                           LSTUPDT_NBR,
                           BASE_RULE_ID,
                           RULE_VERSION_NBR)
						SELECT ALERT_ACTION_SEQ.nextval ,
							L_CURRENT_UTC_TIMESTAMP AS ACTION_TIMESTAMP,
							'AUTO UNCHECKOUT'  AS ACTION_TYPE,
							ALERT_ID, ALERT_VERSION_NBR,PARENT_ALERT_VALUE, PARENT_ALERT_TYPE,ALERT_VALUE,
							CONTACT_VALUE,CONTACT_TYPE,TENANT_MULTI_ORG_ID, ALERT_TYPE, QUEUE_ID,
							MULTI_ORG_ID, RESULT_ID, 'ACTIVE',
							GMT_OFFSET,
							FRAUD_TYPE_CODE,
							FRAUD_SCORE,
							FRAUD_SCORE_REASON_CODE_1, FRAUD_SCORE_REASON_CODE_2,
							FRAUD_SCORE_REASON_CODE_3, INIT_FRAUD_SCORE,
							INIT_FRAUD_SCORE_REASON_CODE_1,
							INIT_FRAUD_SCORE_REASON_CODE_2,
							INIT_FRAUD_SCORE_REASON_CODE_3, HWM_FRAUD_SCORE,
							HWM_FRAUD_SCORE_REASON_CODE_1,
							HWM_FRAUD_SCORE_REASON_CODE_2,
							HWM_FRAUD_SCORE_REASON_CODE_3, ALERT_SERVICE_CNT,
							ALERT_ORIGIN, INIT_TRANSACTION_AMT,
							INIT_TRANSACTION_TIMESTAMP, LST_TRANSACTION_AMT,
							LST_TRANSACTION_TIMESTAMP, USER_SERVICE_TIMESTAMP,
							SYSTEM_SERVICE_TIMESTAMP, NEXT_SERVICE_TIMESTAMP,
							QUEUE_ENTRY_TIMESTAMP, FINAL_DISPOSITION_TIMESTAMP,
							CHECKOUT_USER, CHECKOUT_TIMESTAMP, CREATE_USER,
							CREATE_TIMESTAMP, LSTUPDT_USER, LSTUPDT_TIMESTAMP,
						  LSTUPDT_NBR, BASE_RULE_ID, RULE_VERSION_NBR
						FROM FCM_ALERT
						WHERE ALERT_ID = I_ALERT_ID;

			
						-- step 3. update alerts unchecking them out (status = active)
						UPDATE FCM_ALERT
						SET ALERT_STATUS = 'ACTIVE',
						  CHECKOUT_USER = NULL,
						  CHECKOUT_TIMESTAMP = NULL,
						  LSTUPDT_USER = I_USER_NAME,
						  LSTUPDT_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP,
						  LSTUPDT_NBR = LSTUPDT_NBR + 1
						WHERE ALERT_ID = I_ALERT_ID;

/* commented out for race condition*/
 						Select CONTACT_VALUE,CONTACT_TYPE,TENANT_MULTI_ORG_ID into L_CONTACT_VALUE, l_contact_Type,L_TENANT_MULTI_ORG_ID from fcm_alert where alert_id =  I_ALERT_ID;
                        /* Have to use max, because if no records, or alert_suspend is not on, it will error out*/	            	
			            Select coalesce(max(ALERT_CHECKOUT_CNT),0) into L_CNTR from fcm_alert_suspend
			            	where CONTACT_VALUE = L_CONTACT_VALUE
									and contact_type = l_contact_Type
									and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						If (L_CNTR = 1) then
							delete from fcm_alert_suspend
								where CONTACT_VALUE = L_CONTACT_VALUE
									and contact_type = l_contact_Type
									and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						else if (L_CNTR > 1) THEN
							update fcm_alert_suspend
						  		set ALERT_CHECKOUT_CNT = ALERT_CHECKOUT_CNT -1
						  		where CONTACT_VALUE = L_CONTACT_VALUE
									and contact_type = l_contact_Type
									and TENANT_multi_org_id = l_TENANT_multi_org_id;-- and suspend_user = I_USER_NAME;
						  end if;
						end if;
			 --						DELETE FROM FCM_ALERT_SUSPEND WHERE ALERT_ID = I_ALERT_ID;
						-- step 4. commit work;
						COMMIT;

END UNCHECKOUT_SPECIFIC_ALERT;

/
show errors;


