
  CREATE OR REPLACE PROCEDURE UNPAUSE_ALERT (
     i_user_name IN varchar2,
     i_alert_id IN number,
     i_lstupdt_nbr IN number,
     o_lstupdt_nbr OUT number,
     o_sqlcode OUT number,
     o_sqlstate OUT varchar2 )

-----------------------------------------------------------------------------
-- file:  unpause_alert
--
--
-- description:
--    allows the analyst to resume a timer after pausing an alert.
--
--
-- called by:
--     Web UI
--
-- calls:
--     get_alert_row
--     save_alert_action
--
-- notes:
--
--
-- usage:
--
--
-- history:
--     2012/07/23    leushe				 Ported from DB2
--     2013/03/12    leushe      Change support to alert/entity values of 100 bytes
--     2014/02/26    leushe              Corrected the timestamp calculation to get seconds
------------------------------------------------------------------------
IS

     l_rowcount integer default -1;



     l_alrt_ver_nbr integer;
     l_alert_value  VARCHAR2(100) ;
     L_PARENT_ALERT_VALUE  varchar2(100);
     L_CONTACT_VALUE  varchar2(100);
     L_PARENT_ALERT_TYPE  varchar2(1);
     L_CONTACT_TYPE  varchar2(1);
     l_TENANT_multi_org_id integer;
     l_queue_id number ;
     l_multi_org_id integer;
     l_alert_type VARCHAR2(1);
     l_result_id  integer;
     l_alert_status VARCHAR2(10) ;
     l_gmt_offset number DEFAULT 0.0;
     l_fraud_type_code  VARCHAR2(3);
     l_pend_reason_code VARCHAR2(4);
     l_fraud_score  integer;
     l_fraud_score_reason_code_1  integer;
     l_fraud_score_reason_code_2   integer;
     l_fraud_score_reason_code_3   integer;
     l_init_fraud_score integer;
     l_init_frd_score_reason_code_1  integer;
     l_init_frd_score_reason_code_2  integer;
     l_init_frd_score_reason_code_3  integer;
     l_hwm_fraud_score  integer;
     l_hwm_frd_score_reason_code_1   integer;
     l_hwm_frd_score_reason_code_2   integer;
     l_hwm_frd_score_reason_code_3   integer;
     l_alert_service_cnt  integer  ;
     l_alert_origin VARCHAR2(20) ;
     l_lst_transaction_amt  BINARY_DOUBLE;
     l_lst_transaction_timestamp  timestamp;
     l_user_service_timestamp timestamp;
     l_system_service_timestamp timestamp;
     l_next_service_timestamp timestamp;
     l_queue_entry_timestamp  timestamp;
     l_final_disposition_timestamp  timestamp;
     l_checkout_user  VARCHAR2(60) ;
     l_checkout_timestamp timestamp;
     l_create_user  VARCHAR2(60) ;
     l_create_timestamp timestamp;
     l_lstupdt_user VARCHAR2(60);
     l_lstupdt_timestamp  timestamp;
     l_lstupdt_nbr  integer;
     l_base_rule_id number;
     l_rule_version_nbr integer;
     l_strategy_id  number;


     l_service_method_code VARCHAR2(4);
     l_action_type VARCHAR2(20) default 'ANALYST ACTION';
     l_pause_timestamp timestamp;
     l_pause_elapsed_time integer;


BEGIN




    update fcm_alert
      set lstupdt_timestamp = SYS_EXTRACT_UTC(CURRENT_TIMESTAMP),
          lstupdt_nbr = lstupdt_nbr + 1,
          lstupdt_user = i_user_name,
          pause_timestamp = null,
          pause_elapsed_time =  abs(extract(second from (pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP))) + extract(minute from (pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)))*60 + extract(hour from (pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)))*60*60 + extract(day from (pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)))*24*60*60)
          --substr((pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)),instr((pause_timestamp-SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)),' ')+7,2) +pause_elapsed_time
    where alert_id = i_alert_id
      and lstupdt_nbr = i_lstupdt_nbr
       and CHECK_USR_MO_AT_ACCESS(i_user_name,multi_org_id,alert_type) = 1
    ;

	l_rowcount := SQL%ROWCOUNT;

    if ( l_rowcount = 1 ) then

       --
       -- insert an alert_action row ?
       --


        get_alert_row
         ( i_alert_id
          ,l_alrt_ver_nbr
          ,l_alert_value
          ,l_parent_alert_value
          ,l_parent_alert_type
          ,l_contact_value
          ,l_contact_type         
          ,l_TENANT_multi_org_id
          ,l_queue_id
          ,l_multi_org_id
          ,l_alert_type
          ,l_result_id
          ,l_alert_status
          ,l_gmt_offset
          ,l_fraud_type_code
          ,l_fraud_score
          ,l_fraud_score_reason_code_1
          ,l_fraud_score_reason_code_2
          ,l_fraud_score_reason_code_3
          ,l_init_fraud_score
          ,l_init_frd_score_reason_code_1
          ,l_init_frd_score_reason_code_2
          ,l_init_frd_score_reason_code_3
          ,l_hwm_fraud_score
          ,l_hwm_frd_score_reason_code_1
          ,l_hwm_frd_score_reason_code_2
          ,l_hwm_frd_score_reason_code_3
          ,l_alert_service_cnt
          ,l_alert_origin
          ,l_lst_transaction_amt
          ,l_lst_transaction_timestamp
          ,l_user_service_timestamp
          ,l_system_service_timestamp
          ,l_next_service_timestamp
          ,l_queue_entry_timestamp
          ,l_final_disposition_timestamp
          ,l_checkout_user
          ,l_checkout_timestamp
          ,l_create_user
          ,l_create_timestamp
          ,l_lstupdt_user
          ,l_lstupdt_timestamp
          ,l_lstupdt_nbr
          ,l_base_rule_id
          ,l_rule_version_nbr
          ,l_pause_timestamp
          ,l_pause_elapsed_time
          );

        save_alert_action
	    ( i_alert_id
	     ,l_alrt_ver_nbr
	     ,l_alert_value
         ,l_parent_alert_value
         ,l_parent_alert_type
         ,l_contact_value
         ,l_contact_type
         ,l_TENANT_multi_org_id
	     ,l_queue_id
	     ,l_multi_org_id
	     ,l_alert_type
	     ,null                 -- 2006/06/05 change  ,l_result_id    always null here
	     ,l_alert_status
	     ,l_gmt_offset
	     ,l_fraud_type_code
	     ,l_fraud_score
	     ,l_fraud_score_reason_code_1
	     ,l_fraud_score_reason_code_2
	     ,l_fraud_score_reason_code_3
	     ,l_init_fraud_score
	     ,l_init_frd_score_reason_code_1
	     ,l_init_frd_score_reason_code_2
	     ,l_init_frd_score_reason_code_3
	     ,l_hwm_fraud_score
	     ,l_hwm_frd_score_reason_code_1
	     ,l_hwm_frd_score_reason_code_2
	     ,l_hwm_frd_score_reason_code_3
	     ,l_alert_service_cnt
	     ,l_alert_origin
	     ,l_lst_transaction_amt
	     ,l_lst_transaction_timestamp
	     ,l_user_service_timestamp
	     ,l_system_service_timestamp
	     ,l_next_service_timestamp
	     ,l_queue_entry_timestamp
	     ,l_final_disposition_timestamp
	     ,i_user_name
	     ,l_checkout_timestamp
	     ,l_create_user
	     ,l_create_timestamp
	     ,i_user_name --,l_lstupdt_user
	     ,l_lstupdt_timestamp
	     ,l_lstupdt_nbr
          --   ,null --,l_pend_reason_code
             ,null --,l_reroute_queue_id
             ,l_service_method_code
             ,l_action_type
             ,'N' -- l_need_to_commit
             ,l_base_rule_id
             ,l_rule_version_nbr
             ,l_pause_timestamp
             ,l_pause_elapsed_time
             ,null --,l_strategy_id
             );


       commit;
       o_sqlcode := sqlcode;
       o_sqlstate := substr(SQLERRM,1,5);
       o_lstupdt_nbr := i_lstupdt_nbr + 1;
    else
       rollback;
       o_sqlcode := -1;
       o_sqlstate := 'ERROR';
    end if;

end unpause_alert;

/
show errors;


