CREATE OR REPLACE PROCEDURE QRY_ALERT_SERVICE (
     I_USER_ID IN  NUMBER,
     I_USER_NAME    IN VARCHAR2,
     I_STRATEGY_ID IN    NUMBER,
     O_ALERT_ID    OUT NUMBER )

-----------------------------------------------------------------------------
-- File:  QRY_ALERT_SERVICE
--
--
-- Description:
--
--
-- Called By:
--     GET_NEXT_ALERT
--
-- Calls:
--     GET_ALERT_FROM_QUEUE
--
-- Notes:
--            This is the R3 version of qry_alert_service.
--
--
-- Usage:
--
--
-- History:
--     2012/07/23    leushe                 Ported from DB2
--     2014/07/23    leushe                 Changed the query to not call CHECK_MO_AT_ACCESS function
--                                          Changed the cursor select to return the upper/lower hours and superseded indicator to 
--                                          be passed into the GET_NEXT_ALERT proc
--
--
------------------------------------------------------------------------
is
   SQLSTATE CHAR(5);
   l_index INT;
   ii INT;
   l_alert_id NUMBER;
   l_rowcount INT DEFAULT 0;
   l_rowindex INT;
   l_queue_id NUMBER;
   l_queue_sort_field VARCHAR2(30);
   l_queue_sort_order VARCHAR2(10);
   l_queue_call_superseded_ind varchar2(1);
   l_queue_call_lower_bound number(5);
   l_queue_call_upper_bound number(5);



  CURSOR cursor1   IS
     WITH
                uat as (select alert_type from fcm_user_alert_type where user_id= i_user_id),
                umo as (select multi_org_id from fcm_user_multi_org where user_id= i_user_id)
       select q.queue_id, q.queue_sort_order, q.queue_sort_field,q.queue_call_superseded_ind,q.queue_call_lower_bound,q.queue_call_upper_bound
       from fcm_queue q, fcm_strategy_user su
       where q.strategy_id = i_strategy_id and
             q.queue_status in ('ACTIVE', 'QUIESCE') and
             su.user_id = i_user_id and
             su.strategy_id = q.strategy_id and
             q.logical_del_ind = '0' and
             queue_id =
             any (
                select a.queue_id
                from fcm_alert a, uat, umo
                where a.queue_id = q.queue_id and
                      a.multi_org_id=umo.multi_org_id and
                      a.alert_type=uat.alert_Type and
                      a.alert_status='ACTIVE' and
                      a.next_service_timestamp < SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) and
                      ( (  curr_cust_hour(a.gmt_offset ) between
                      q.queue_call_lower_bound and q.queue_call_upper_bound ) or
                      (q.queue_call_superseded_ind = '1') )
                      and not exists (select 1 from fcm_alert_suspend fas
                        where fas.contact_value = a.contact_value
                        and fas.contact_type = a.contact_type
                        and a.TENANT_multi_org_id = fas.TENANT_multi_org_id
                        and fas.suspend_user <> i_user_name
                        and a.contact_value is not null)
                        and rownum = 1
             )
             order by q.queue_priority asc
             ;

BEGIN




   open cursor1;

   FETCH  cursor1 into
          l_queue_id, l_queue_sort_order, l_queue_sort_field,l_queue_call_superseded_ind,l_queue_call_lower_bound,l_queue_call_upper_bound;

   close cursor1;

   IF ( l_queue_id is NULL ) THEN -- no queues left in strategy
       o_alert_id := -2;
      RETURN;
   END IF;

   -- returns -1 if no alert found available for servicing in the queue
    GET_ALERT_FROM_QUEUE (I_USER_ID,i_user_name, l_queue_id, l_queue_sort_field, l_queue_sort_order,l_queue_call_superseded_ind,l_queue_call_lower_bound,l_queue_call_upper_bound, l_alert_id);


    o_alert_id := l_alert_id;

END QRY_ALERT_SERVICE;
/
show errors;